/*
 * util.h - header file for utility functions.
 *
 * Copyright (C) 2023 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef UTIL_H
#define UTIL_H


#include <libubox/list.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <easy/easy.h>



void do_daemonize(const char *pidfile);

typedef unsigned int ifstatus_t;
typedef unsigned char ifopstatus_t;

int if_brportnum(const char *ifname);
int if_getcarrier(const char *ifname, int *carrier);

#ifndef BIT
#define BIT(x)	(1 << (x))
#endif

#define MACFMT		"%02x:%02x:%02x:%02x:%02x:%02x"
#define MAC2STR(_m)	(_m)[0], (_m)[1], (_m)[2], (_m)[3], (_m)[4], (_m)[5]



#ifndef list_flush
#define list_flush(head, type, member)					\
do {									\
	type *__p, *__tmp;						\
									\
	if (!list_empty(head))						\
		list_for_each_entry_safe(__p, __tmp, head, member) {	\
			list_del(&__p->member);				\
			free(__p);					\
		}							\
} while (0)
#endif

#define foreach_token_r(t, tokens, saveptr, delim)	\
	for ((t = strtok_r(tokens, delim, &saveptr)); t; (t = strtok_r(NULL, delim, &saveptr)))

#endif /* UTIL_H */
